/*
 * GIMP ColorDialog Plug-in -- This is a plug-in for The GIMP 2.2
 * Copyright (C) 2007 keines, <keines.2007@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

#include <windows.h>
#include <libgimp/gimp.h>

static void init  (void);
static void query (void);
static void run   (const gchar      *name,
                   gint              nparams,
                   const GimpParam  *param,
                   gint             *nreturn_vals,
                   GimpParam       **return_vals);

#define DEF_BACK "Back - "
#define DEF_FORE "Fore - "
#define DEF_LEN strlen(DEF_BACK)

#define WINAPI_COLORDIALOG_PALETTES "Plug_in_WinAPI_ColorDialog_Palettes"
struct PalettesValues {
  COLORREF C[16] ; // 0..15
};

#define WINAPI_COLORDIALOG_CONDITION "Plug_in_WinAPI_ColorDialog_CONDITION"
struct ConditionValue {
  int   x,y ;
  guchar   IsForeground ;
};

// Ăяo֐̓o^
GimpPlugInInfo PLUG_IN_INFO =
{
  init,  // init_proc
  NULL,  // quit_proc
  query, // query_proc
  run,   // run_proc
};

MAIN ()

static void
init (void) {
  struct PalettesValues   Palettes;
  struct ConditionValue  Condition;

  gimp_get_data(WINAPI_COLORDIALOG_CONDITION, &Condition);
    Condition.IsForeground = 1 ;
    Condition.x = -1 ;
    Condition.y = 0 ;
  gimp_set_data(WINAPI_COLORDIALOG_CONDITION, &Condition, sizeof(struct ConditionValue));

  gimp_get_data(WINAPI_COLORDIALOG_PALETTES, &Palettes);
    int i ;
    for (i=0; i<16; i++) Palettes.C[i]=0xFFFFFF ;
  gimp_set_data(WINAPI_COLORDIALOG_PALETTES, &Palettes, sizeof(struct PalettesValues));
}

static void query (void) {
    static GimpParamDef args[] = {
      { GIMP_PDB_INT32,"run-mode", "Run mode" },	// 0 ̎Oނ͕K{炵
      { GIMP_PDB_IMAGE,"image","Input image" },		// 1
      { GIMP_PDB_DRAWABLE,"drawable","Input drawable"}	// 2
    };

  gimp_install_procedure (
    "plug-in-ColorDialog",	// ֐
    "WinAPI ChooseColor",	// 
    "Windows 9x/ME/XP Only.\nX key Change to Foreground or Background Color.\nCtrl Key Set to Color.",// ǉ
    "Keina",			// ҏ
    "k.keina",			//
    "2007.09.29(Sat) 11:55",	//
    "_ColorDialog...",		// j[ɕ\镶 _ V[gJbgL[ɂȂ
    "RGB*, GRAY*, INDEXED*",	// pł摜[h炵
    GIMP_PLUGIN,		// s[h炵
    G_N_ELEMENTS (args), 0 ,	// TCY
    args, NULL);

  // j[ւ̓o^
  gimp_plugin_menu_register ("plug-in-ColorDialog","<Image>/Filters/Misc");
}

// F_CAO̖OύX
void SetConditionText (HWND hdlg , guchar IsChange) {
  struct ConditionValue  Condition;

  //gimp_context_swap_colors () ;
  gimp_get_data(WINAPI_COLORDIALOG_CONDITION, &Condition);
  if (IsChange==1) {
    Condition.IsForeground=!(Condition.IsForeground==1);
    gimp_set_data(WINAPI_COLORDIALOG_CONDITION, &Condition, sizeof(struct ConditionValue));
  }

  int len=GetWindowTextLength(hdlg)+2 ;
  int len2;
  char sTitle[len];				// ̃^Cg
  char s1[2][DEF_LEN+1] = {DEF_BACK,DEF_FORE};	// ǉ{ +1  \0
  char s2[DEF_LEN] ;				// 

  // ̃^Cg擾
  GetWindowText(hdlg , sTitle , len);
  //printf ("%s,%d\n",sTitle,strlen(sTitle));
  
  // s2 Ƀ^Cg̐擪擾
  strncpy(s2,sTitle,DEF_LEN);

  // ^Cg̕ύX`FbN
  if ((strncmp(s2,s1[0],DEF_LEN)==0) || (strncmp(s2,s1[1],DEF_LEN)==0)) {
    // ǉ
    len=DEF_LEN;
    len2=strlen(sTitle)-DEF_LEN;
  }else{
    // ǉȂ
    len=0;
    len2=strlen(sTitle);
  }

  // len  : ύXÕ^Cg̃obt@ʒu
  // len2 : ^Cgݒׂ̈ɕKvɂȂ钷

  s2[len2+strlen(s1[Condition.IsForeground])+2] ;

  // s2  Fore / Back 
  strcpy (s2, s1[Condition.IsForeground]);
  // s2 Ƀ^Cgǉ
  strcat (s2, sTitle+len);

  // ̃^CgύX
  SetWindowText(hdlg , s2);
  return ;
}

void SendColor (COLORREF c) {
  struct ConditionValue  Condition;
  GimpRGB col ;
  guchar r,g,b,a ;

  // bbggrr
  r = (c & 0x0000ff) ;
  g = (c & 0x00ff00) >> 8 ;
  b = (c & 0xff0000) >>16 ;

  //printf ("%d,%d,%d\n",r,g,b);
  
  // 0..255 -> 0..1.0
  gimp_rgba_set_uchar(&col,r,g,b,a);

  // Fore / Back
  gimp_get_data(WINAPI_COLORDIALOG_CONDITION, &Condition);
  if (Condition.IsForeground==1) {
    gimp_context_set_foreground (&col);
  }else{
    gimp_context_set_background (&col);
  }
}

UINT CALLBACK CCHookProc(HWND hdlg , UINT uiMsg , WPARAM wp , LPARAM lp) {
  struct ConditionValue  Condition;
  POINT pt ;
  RECT  rc ;
  HWND hWork ;
  HDC hdc ;
  COLORREF c ;
  switch(uiMsg) {
    case WM_INITDIALOG:
      gimp_get_data(WINAPI_COLORDIALOG_CONDITION, &Condition);
      if (Condition.x==-1) {
        pt.x = GetSystemMetrics(SM_CXSCREEN);  // ݂̃fBXvC̑傫擾
        pt.y = GetSystemMetrics(SM_CYSCREEN);  //
        GetWindowRect(hdlg,&rc);
        ClientToScreen(0,&pt);
        pt.x=(pt.x-(rc.right-rc.left)) >> 1 ;
        pt.y=(pt.y-(rc.bottom-rc.top)) >> 1 ;
      }else{
        pt.x=Condition.x;
        pt.y=Condition.y;
      }
      // ʊOɏoƂ͍Xɕ␳KvA
      // {p^[ƂĂ ColorDialogPlus ƈꏏɎgƂlĂ̂
      // W␳͂Ɋۓ
      SetWindowPos(hdlg,HWND_TOPMOST,pt.x, pt.y,0,0,SWP_NOSIZE);
      SetForegroundWindow(hdlg);
    
      SetConditionText(hdlg,0);
      return 1 ; // _CAOɑ΂ĂȂ񂩂True
    
    case WM_KEYDOWN:
    //case  WM_KEYUP:
      //if (wp=="D") { 
    	switch (wp) {
          // Foreground / Background
          case 'X':
          SetConditionText(hdlg,1);
          return 1 ;

          // FGIMPɑ
          case VK_CONTROL:
          hWork = GetDlgItem(hdlg,709) ;
	  hdc = GetDC(hWork) ;
	    c = GetPixel(hdc, 10, 10) ;
          DeleteDC(hdc);
          SendColor(c);
          break ;//return 1 ;
        }
    
    case WM_DESTROY:
      gimp_get_data(WINAPI_COLORDIALOG_CONDITION, &Condition);
        GetWindowRect(hdlg,&rc);
        //ClientToScreen(0,&pt);
        Condition.x=rc.left;
        Condition.y=rc.top;
      gimp_set_data(WINAPI_COLORDIALOG_CONDITION, &Condition, sizeof(struct ConditionValue));
      return 0 ;

  }
  return 0 ;
}

static void
run (const gchar      *name,
     gint              nparams,		// 炭
     const GimpParam  *param,		// ͒l
     gint             *nreturn_vals,	// 
     GimpParam       **return_vals)	// Ԓl
{
  // + O
  static GimpParam  values[1];
  struct PalettesValues  Palettes;
  GimpPDBStatusType status = GIMP_PDB_SUCCESS;
  GimpRunMode       run_mode;

  // + Ԃl̐ݒ
  *nreturn_vals = 1;			// 
  *return_vals  = values;		// l
  values[0].type          = GIMP_PDB_STATUS;
  values[0].data.d_status = GIMP_PDB_EXECUTION_ERROR;
  
  // + C
  run_mode = param[0].data.d_int32;
  switch (run_mode) {
    case GIMP_RUN_INTERACTIVE:
    case GIMP_RUN_WITH_LAST_VALS:
      //gimp_ui_init ("_ColorDialog", FALSE);
      gimp_get_data(WINAPI_COLORDIALOG_PALETTES, &Palettes);

      // + OF 
      guchar r,g,b,a ;
      GimpRGB col ;
      gimp_context_get_foreground (&col);    // 0.0 .. 1.0
      gimp_rgba_get_uchar(&col,&r,&g,&b,&a); // 0.0 .. 1.0 -> 0..255

      static CHOOSECOLOR cc = {0};
      cc.lStructSize	= sizeof (CHOOSECOLOR);
      cc.hwndOwner	= 0 ;
      cc.rgbResult	= RGB(r,g,b) ;
      cc.lpCustColors	= Palettes.C ;
      cc.Flags		= CC_RGBINIT | CC_FULLOPEN | CC_ENABLEHOOK ;
      cc.lpfnHook	= CCHookProc ;
      // Ctrl Ȃ̋N͔wFɐݒ肷
      // ʏ͑SʐF 
      //int IsControl = (GetKeyState(VK_CONTROL) && 0x8000) ;

      if (ChooseColor(&cc)) {
        SendColor(cc.rgbResult);
        //gimp_displays_flush();
      }else{
        status = GIMP_PDB_CANCEL ;
      }
      gimp_set_data(WINAPI_COLORDIALOG_PALETTES, &Palettes, sizeof(struct PalettesValues));
    break;
/*
    case GIMP_RUN_NONINTERACTIVE:
      // XXX FIXME: params_to_do_nothing_values ()
      // do_nothing()
      break;
    default:
      break;
*/
  }

  // + ㏈
  values[0].data.d_status = status ;
  return ;
}
